package DBIx::Class::ResultSet::WithMetaData;

use strict;
use warnings;

use Data::Alias;
use Moose;
use MooseX::Method::Signatures;
extends 'DBIx::Class::ResultSet';

has '_row_info' => (
	is => 'rw',
	isa => 'HashRef'
);

has 'was_row' => (
	is => 'rw',
	isa => 'Int'
);

sub new {
	my $self = shift;
  my $new = $self->next::method(@_);
	foreach my $key (qw/_row_info/) {
		alias $new->{$key} = $new->{attrs}{$key};
		$new->{$key} = {} unless $new->{$key};
	}

	return $new;
}

method with_token {
	foreach my $row ($self->all) {
		my $token = $self->tokenify($row->get_column($self->token_col));
		$self->add_row_info(id => $row->id, info => { join('_', 'token', $self->token_col) => $token });
	}

	return $self;
}

method display () {
  my $rs = $self->search({});
  $rs->result_class('DBIx::Class::ResultClass::HashRefInflator');
  my @rows;
	foreach my $row ($rs->all) {
		if (my $info = $self->row_info_for(id => $row->{id})) {
			$row = { %{$row}, %{$info} };
		}
		push(@rows, $row);
	}

  return ($self->was_row) ? $rows[0] : \@rows;
}

method add_row_info (Int :$id, HashRef :$info) {
	unless ($self->find($id)) {
		warn $id;
		die 'invalid id passed to add_row_info';
	}

	if (my $existing = $self->_row_info->{$id}) {
		$info = { %{$existing}, %{$info} };
	}

	$self->_row_info->{$id} = $info;	
}

method row_info_for (Int :$id) {
	return $self->_row_info->{$id};
}

method order_by (Str :$col) {
	$col = "me.$col" unless ($col =~ m/\./);
	return $self->search({}, { order_by => $col });
}

method limit (Int :$count) {
	return $self->search({}, { rows => $count });
}

sub tokenify {
  my ($self, $string) = @_;

  $string =~ s/\s+$//;
  $string =~ s/[,\.\-\+]//g;
  $string =~ s/^\s+//;
  $string =~ s/ /-/g;
  $string = lc($string);
  return $string;
}

sub clean_rs {
  my $self = shift;

  return $self->result_source->resultset;
}

1;
